@@ -1,5 +1,5 @@
 /*
-  $Id: sample3.c,v 1.5 2003/10/03 08:36:39 rocky Exp $
+  $Id: sample3.c,v 1.6 2003/11/05 04:12:57 rocky Exp $
 
   Copyright (C) 2003 Rocky Bernstein <rocky@panix.com>
   
@@ -32,7 +32,7 @@
 #include <cdio/cd_types.h>
 
 static void
-print_analysis(cdio_analysis_t cdio_analysis, 
+print_analysis(cdio_iso_analysis_t cdio_iso_analysis, 
 	       cdio_fs_anal_t fs, int first_data, unsigned int num_audio, 
 	       track_t num_tracks, track_t first_track_num, CdIo *cdio)
 {
@@ -42,7 +42,7 @@ print_analysis(cdio_analysis_t cdio_analysis,
   case CDIO_FS_ISO_9660:
     printf("CD-ROM with ISO 9660 filesystem");
     if (fs & CDIO_FS_ANAL_JOLIET) {
-      printf(" and joliet extension level %d", cdio_analysis.joliet_level);
+      printf(" and joliet extension level %d", cdio_iso_analysis.joliet_level);
     }
     if (fs & CDIO_FS_ANAL_ROCKRIDGE)
       printf(" and rockridge extensions");
@@ -81,7 +81,7 @@ print_analysis(cdio_analysis_t cdio_analysis,
   case CDIO_FS_ISO_9660_INTERACTIVE:
   case CDIO_FS_ISO_HFS:
     printf("ISO 9660: %i blocks, label `%.32s'\n",
-	   cdio_analysis.isofs_size, cdio_analysis.iso_label);
+	   cdio_iso_analysis.isofs_size, cdio_iso_analysis.iso_label);
     break;
   }
   if (first_data == 1 && num_audio > 0)
@@ -153,9 +153,9 @@ main(int argc, const char *argv[])
   } else {
     /* we have data track(s) */
     int j;
-    cdio_analysis_t cdio_analysis; 
+    cdio_iso_analysis_t cdio_iso_analysis; 
 
-    memset(&cdio_analysis, 0, sizeof(cdio_analysis));
+    memset(&cdio_iso_analysis, 0, sizeof(cdio_iso_analysis));
     
     for (j = 2, i = first_data; i <= num_tracks; i++) {
       lsn_t lsn;
@@ -181,12 +181,12 @@ main(int argc, const char *argv[])
 	data_start = start_track;
       
       /* skip tracks which belong to the current walked session */
-      if (start_track < data_start + cdio_analysis.isofs_size)
+      if (start_track < data_start + cdio_iso_analysis.isofs_size)
 	continue;
       
-      fs = cdio_guess_cd_type(cdio, start_track, i, &cdio_analysis);
+      fs = cdio_guess_cd_type(cdio, start_track, i, &cdio_iso_analysis);
       
-      print_analysis(cdio_analysis, fs, first_data, num_audio,
+      print_analysis(cdio_iso_analysis, fs, first_data, num_audio,
 		     num_tracks, first_track_num, cdio);
       
       if ( !(CDIO_FSTYPE(fs) == CDIO_FS_ISO_9660 ||
