@@ -1,5 +1,5 @@
 /*
-    $Id: _cdio_sunos.c,v 1.15 2003/09/01 15:11:36 rocky Exp $
+    $Id: _cdio_sunos.c,v 1.22 2003/10/14 04:44:52 rocky Exp $
 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
     Copyright (C) 2002,2003 Rocky Bernstein <rocky@panix.com>
@@ -23,19 +23,25 @@
 # include "config.h"
 #endif
 
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+#ifdef HAVE_GLOB_H
+#include <glob.h>
+#endif
 
 #include <cdio/logging.h>
 #include <cdio/sector.h>
 #include <cdio/util.h>
 #include "cdio_assert.h"
 #include "cdio_private.h"
+#include "scsi_mmc.h"
 
 #define DEFAULT_CDIO_DEVICE "/vol/dev/aliases/cdrom0"
 
 #ifdef HAVE_SOLARIS_CDROM
 
-static const char _rcsid[] = "$Id: _cdio_sunos.c,v 1.15 2003/09/01 15:11:36 rocky Exp $";
+static const char _rcsid[] = "$Id: _cdio_sunos.c,v 1.22 2003/10/14 04:44:52 rocky Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -116,14 +122,14 @@ _cdio_init (_img_private_t *_obj)
    Returns 0 if no error. 
  */
 static int
-_cdio_read_mode2_sector (void *user_data, void *data, lsn_t lsn, 
+_cdio_read_mode2_sector (void *env, void *data, lsn_t lsn, 
 			 bool mode2_form2)
 {
   char buf[M2RAW_SECTOR_SIZE] = { 0, };
   struct cdrom_msf *msf = (struct cdrom_msf *) &buf;
   msf_t _msf;
 
-  _img_private_t *_obj = user_data;
+  _img_private_t *_obj = env;
 
   cdio_lba_to_msf (cdio_lsn_to_lba(lsn), &_msf);
   msf->cdmsf_min0 = from_bcd8(_msf.m);
@@ -164,35 +170,21 @@ _cdio_read_mode2_sector (void *user_data, void *data, lsn_t lsn,
       {
 	struct uscsi_cmd sc;
 	union scsi_cdb cdb;
-	int blocks = 1;
-	int sector_type;
-	int sync, header_code, user_data, edc_ecc, error_field;
-	int sub_channel;
-	
-	sector_type = 0;		/* all types */
-	/*sector_type = 1;*/	/* CD-DA */
-	/*sector_type = 2;*/	/* mode1 */
-	/*sector_type = 3;*/	/* mode2 */
-	/*sector_type = 4;*/	/* mode2/form1 */
-	/*sector_type = 5;*/	/* mode2/form2 */
-	sync = 0;
-	header_code = 2;
-	user_data = 1;
-	edc_ecc = 0;
-	error_field = 0;
-	sub_channel = 0;
-	
+	int nblocks = 1;
+	int sector_type = 0;   /* all types */
+	int sync        = 0;
+	int header_code = 2;
+	int sub_channel = 0;
+	int user_data   = 1;
+	int edc_ecc     = 0;
+	int error_field = 0;
+
 	memset(&cdb, 0, sizeof(cdb));
 	memset(&sc, 0, sizeof(sc));
-	cdb.scc_cmd = 0xBE;
-	cdb.cdb_opaque[1] = (sector_type) << 2;
-	cdb.cdb_opaque[2] = (lsn >> 24) & 0xff;
-	cdb.cdb_opaque[3] = (lsn >> 16) & 0xff;
-	cdb.cdb_opaque[4] = (lsn >>  8) & 0xff;
-	cdb.cdb_opaque[5] =  lsn & 0xff;
-	cdb.cdb_opaque[6] = (blocks >> 16) & 0xff;
-	cdb.cdb_opaque[7] = (blocks >>  8) & 0xff;
-	cdb.cdb_opaque[8] =  blocks & 0xff;
+	cdb.scc_cmd = CDIO_MMC_GPCMD_READ_CD;
+	CDIO_MMC_SET_READ_TYPE(cdb.cdb_opaque, sector_type);
+	CDIO_MMC_SET_READ_LBA(cdb.cdb_opaque, lsn);
+	CDIO_MMC_SET_READ_LENGTH(cdb.cdb_opaque, nblocks);
 	cdb.cdb_opaque[9] = (sync << 7) |
 	  (header_code << 5) |
 	  (user_data << 4) |
@@ -228,17 +220,22 @@ _cdio_read_mode2_sector (void *user_data, void *data, lsn_t lsn,
 }
 
 /*!
-   Reads a single audio sector from CD device into data starting from lsn.
+   Reads audio sectors from CD device into data starting from lsn.
    Returns 0 if no error. 
- */
+
+   May have to check size of nblocks. There may be a limit that
+   can be read in one go, e.g. 25 blocks.
+*/
+
 static int
-_cdio_read_audio_sector (void *user_data, void *data, lsn_t lsn)
+_cdio_read_audio_sectors (void *env, void *data, lsn_t lsn, 
+			  unsigned int nblocks)
 {
   char buf[CDIO_CD_FRAMESIZE_RAW] = { 0, };
   struct cdrom_msf *msf = (struct cdrom_msf *) &buf;
   msf_t _msf;
 
-  _img_private_t *_obj = user_data;
+  _img_private_t *_obj = env;
 
   cdio_lba_to_msf (cdio_lsn_to_lba(lsn), &_msf);
   msf->cdmsf_min0   = from_bcd8(_msf.m);
@@ -255,8 +252,7 @@ _cdio_read_audio_sector (void *user_data, void *data, lsn_t lsn)
       || (_obj->gen.ioctls_debugged < (30 * 75)  
 	  && _obj->gen.ioctls_debugged % 75 == 0)
       || _obj->gen.ioctls_debugged % (30 * 75) == 0)
-    cdio_debug ("reading %2.2d:%2.2d:%2.2d",
-	       msf->cdmsf_min0, msf->cdmsf_sec0, msf->cdmsf_frame0);
+    cdio_debug ("reading %d", lsn);
   
   _obj->gen.ioctls_debugged++;
   
@@ -267,11 +263,17 @@ _cdio_read_audio_sector (void *user_data, void *data, lsn_t lsn)
       return 1;
       break;
       
-    case _AM_SUN_CTRL_SCSI:
-      if (ioctl (_obj->gen.fd, CDROMCDDA, &data) == -1) {
-	perror ("ioctl(..,CDROMCDDA,..)");
-	return 1;
-	/* exit (EXIT_FAILURE); */
+    case _AM_SUN_CTRL_SCSI: 
+      {
+	struct cdrom_cdda cdda;
+	cdda.cdda_addr   = lsn;
+	cdda.cdda_length = nblocks;
+	cdda.cdda_data   = (caddr_t) data;
+	if (ioctl (_obj->gen.fd, CDROMCDDA, &cdda) == -1) {
+	    perror ("ioctl(..,CDROMCDDA,..)");
+	    return 1;
+	    /* exit (EXIT_FAILURE); */
+	}
       }
       break;
       
@@ -279,30 +281,20 @@ _cdio_read_audio_sector (void *user_data, void *data, lsn_t lsn)
       {
 	struct uscsi_cmd sc;
 	union scsi_cdb cdb;
-	int blocks = 1;
-	int sector_type;
-	int sync, header_code, user_data, edc_ecc, error_field;
-	int sub_channel;
-	
-	sector_type = 1;
-	sync = 0;
-	header_code = 2;
-	user_data = 1;
-	edc_ecc = 0;
-	error_field = 0;
-	sub_channel = 0;
+	int sector_type = 1;   /* CD-DA */
+	int sync        = 0;
+	int header_code = 2;
+	int user_data   = 1;
+	int edc_ecc     = 0;
+	int error_field = 0;
+	int sub_channel = 0;
 	
 	memset(&cdb, 0, sizeof(cdb));
 	memset(&sc, 0, sizeof(sc));
-	cdb.scc_cmd = 0xBE;
-	cdb.cdb_opaque[1] = (sector_type) << 2;
-	cdb.cdb_opaque[2] = (lsn >> 24) & 0xff;
-	cdb.cdb_opaque[3] = (lsn >> 16) & 0xff;
-	cdb.cdb_opaque[4] = (lsn >>  8) & 0xff;
-	cdb.cdb_opaque[5] =  lsn & 0xff;
-	cdb.cdb_opaque[6] = (blocks >> 16) & 0xff;
-	cdb.cdb_opaque[7] = (blocks >>  8) & 0xff;
-	cdb.cdb_opaque[8] =  blocks & 0xff;
+	cdb.scc_cmd = CDIO_MMC_GPCMD_READ_CD;
+	CDIO_MMC_SET_READ_TYPE(cdb.cdb_opaque, sector_type);
+	CDIO_MMC_SET_READ_LBA(cdb.cdb_opaque, lsn);
+	CDIO_MMC_SET_READ_LENGTH(cdb.cdb_opaque, nblocks);
 	cdb.cdb_opaque[9] = (sync << 7) |
 	  (header_code << 5) |
 	  (user_data << 4) |
@@ -339,11 +331,11 @@ _cdio_read_audio_sector (void *user_data, void *data, lsn_t lsn)
    Returns 0 if no error. 
  */
 static int
-_cdio_read_mode2_sectors (void *user_data, void *data, uint32_t lsn, 
+_cdio_read_mode2_sectors (void *env, void *data, uint32_t lsn, 
 			  bool mode2_form2, unsigned int nblocks)
 {
-  _img_private_t *_obj = user_data;
-  int i;
+  _img_private_t *_obj = env;
+  unsigned int i;
   int retval;
 
   for (i = 0; i < nblocks; i++) {
@@ -368,9 +360,9 @@ _cdio_read_mode2_sectors (void *user_data, void *data, uint32_t lsn,
    Return the size of the CD in logical block address (LBA) units.
  */
 static uint32_t 
-_cdio_stat_size (void *user_data)
+_cdio_stat_size (void *env)
 {
-  _img_private_t *_obj = user_data;
+  _img_private_t *_obj = env;
 
   struct cdrom_tocentry tocent;
   uint32_t size;
@@ -389,12 +381,17 @@ _cdio_stat_size (void *user_data)
 }
 
 /*!
-  Set the key "arg" to "value" in source device.
+  Set the arg "key" with "value" in the source device.
+  Currently "source" and "access-mode" are valid keys.
+  "source" sets the source device in I/O operations 
+  "access-mode" sets the the method of CD access 
+
+  0 is returned if no error was found, and nonzero if there as an error.
 */
 static int
-_cdio_set_arg (void *user_data, const char key[], const char value[])
+_cdio_set_arg (void *env, const char key[], const char value[])
 {
-  _img_private_t *_obj = user_data;
+  _img_private_t *_obj = env;
 
   if (!strcmp (key, "source"))
     {
@@ -412,7 +409,7 @@ _cdio_set_arg (void *user_data, const char key[], const char value[])
       else if (!strcmp(value, "SCSI"))
 	_obj->access_mode = _AM_SUN_CTRL_SCSI;
       else
-	cdio_error ("unknown access type: %s. ignored.", value);
+	cdio_warn ("unknown access type: %s. ignored.", value);
     }
   else 
     return -1;
@@ -469,9 +466,9 @@ _cdio_read_toc (_img_private_t *_obj)
   also free obj.
  */
 static int 
-_cdio_eject_media (void *user_data) {
+_cdio_eject_media (void *env) {
 
-  _img_private_t *_obj = user_data;
+  _img_private_t *_obj = env;
   int ret;
 
   close(_obj->gen.fd);
@@ -508,9 +505,9 @@ _cdio_malloc_and_zero(size_t size) {
   Return the value associated with the key "arg".
 */
 static const char *
-_cdio_get_arg (void *user_data, const char key[])
+_cdio_get_arg (void *env, const char key[])
 {
-  _img_private_t *_obj = user_data;
+  _img_private_t *_obj = env;
 
   if (!strcmp (key, "source")) {
     return _obj->gen.source_name;
@@ -528,7 +525,7 @@ _cdio_get_arg (void *user_data, const char key[])
 }
 
 /*!
-  Return a string containing the default VCD device if none is specified.
+  Return a string containing the default CD device if none is specified.
  */
 char *
 cdio_get_default_device_solaris(void)
@@ -563,9 +560,9 @@ cdio_get_default_device_solaris(void)
   CDIO_INVALID_TRACK is returned on error.
 */
 static track_t
-_cdio_get_first_track_num(void *user_data) 
+_cdio_get_first_track_num(void *env) 
 {
-  _img_private_t *_obj = user_data;
+  _img_private_t *_obj = env;
   
   if (!_obj->gen.toc_init) _cdio_read_toc (_obj) ;
 
@@ -576,9 +573,9 @@ _cdio_get_first_track_num(void *user_data)
   Return the number of tracks in the current medium.
 */
 static track_t
-_cdio_get_num_tracks(void *user_data) 
+_cdio_get_num_tracks(void *env) 
 {
-  _img_private_t *_obj = user_data;
+  _img_private_t *_obj = env;
   
   if (!_obj->gen.toc_init) _cdio_read_toc (_obj) ;
 
@@ -589,9 +586,9 @@ _cdio_get_num_tracks(void *user_data)
   Get format of track. 
 */
 static track_format_t
-_cdio_get_track_format(void *user_data, track_t track_num) 
+_cdio_get_track_format(void *env, track_t track_num) 
 {
-  _img_private_t *_obj = user_data;
+  _img_private_t *_obj = env;
   
   if (!_obj->gen.init) _cdio_init(_obj);
   if (!_obj->gen.toc_init) _cdio_read_toc (_obj) ;
@@ -623,9 +620,9 @@ _cdio_get_track_format(void *user_data, track_t track_num)
   FIXME: there's gotta be a better design for this and get_track_format?
 */
 static bool
-_cdio_get_track_green(void *user_data, track_t track_num) 
+_cdio_get_track_green(void *env, track_t track_num) 
 {
-  _img_private_t *_obj = user_data;
+  _img_private_t *_obj = env;
   
   if (!_obj->gen.init) _cdio_init(_obj);
   if (!_obj->gen.toc_init) _cdio_read_toc (_obj) ;
@@ -649,9 +646,9 @@ _cdio_get_track_green(void *user_data, track_t track_num)
   False is returned if there is no entry.
 */
 static bool
-_cdio_get_track_msf(void *user_data, track_t track_num, msf_t *msf)
+_cdio_get_track_msf(void *env, track_t track_num, msf_t *msf)
 {
-  _img_private_t *_obj = user_data;
+  _img_private_t *_obj = env;
 
   if (NULL == msf) return false;
 
@@ -682,6 +679,34 @@ cdio_get_default_device_solaris(void)
 }
 #endif /* HAVE_SOLARIS_CDROM */
 
+/*!
+  Return an array of strings giving possible CD devices.
+ */
+char **
+cdio_get_devices_solaris (void)
+{
+#ifndef HAVE_SOLARIS_CDROM
+  return NULL;
+#else
+  char **drives = NULL;
+  unsigned int num_files=0;
+#ifdef HAVE_GLOB_H
+  unsigned int i;
+  glob_t globbuf;
+  globbuf.gl_offs = 0;
+  glob("/vol/dev/aliases/cdrom*", GLOB_DOOFFS, NULL, &globbuf);
+  for (i=0; i<globbuf.gl_pathc; i++) {
+    cdio_add_device_list(&drives, globbuf.gl_pathv[i], &num_files);
+  }
+  globfree(&globbuf);
+#else
+  cdio_add_device_list(&drives, DEFAULT_CDIO_DEVICE, &num_files);
+#endif /*HAVE_GLOB_H*/
+  cdio_add_device_list(&drives, NULL, &num_files);
+  return drives;
+#endif /*HAVE_SOLARIS_CDROM*/
+}
+
 /*!
   Initialization routine. This is the only thing that doesn't
   get called via a function pointer. In fact *we* are the
@@ -699,8 +724,10 @@ cdio_open_solaris (const char *source_name)
     .eject_media        = _cdio_eject_media,
     .free               = cdio_generic_free,
     .get_arg            = _cdio_get_arg,
+    .get_devices        = cdio_get_devices_solaris,
     .get_default_device = cdio_get_default_device_solaris,
     .get_first_track_num= _cdio_get_first_track_num,
+    .get_mcn            = NULL,
     .get_num_tracks     = _cdio_get_num_tracks,
     .get_track_format   = _cdio_get_track_format,
     .get_track_green    = _cdio_get_track_green,
@@ -708,7 +735,7 @@ cdio_open_solaris (const char *source_name)
     .get_track_msf      = _cdio_get_track_msf,
     .lseek              = cdio_generic_lseek,
     .read               = cdio_generic_read,
-    .read_audio_sector  = _cdio_read_audio_sector,
+    .read_audio_sectors = _cdio_read_audio_sectors,
     .read_mode2_sector  = _cdio_read_mode2_sector,
     .read_mode2_sectors = _cdio_read_mode2_sectors,
     .stat_size          = _cdio_stat_size,
