@@ -1,5 +1,5 @@
 /*
-    $Id: _cdio_win32.c,v 1.17 2003/10/17 02:25:36 rocky Exp $
+    $Id: _cdio_win32.c,v 1.20 2003/10/20 04:29:17 rocky Exp $
 
     Copyright (C) 2003 Rocky Bernstein <rocky@panix.com>
 
@@ -26,7 +26,7 @@
 # include "config.h"
 #endif
 
-static const char _rcsid[] = "$Id: _cdio_win32.c,v 1.17 2003/10/17 02:25:36 rocky Exp $";
+static const char _rcsid[] = "$Id: _cdio_win32.c,v 1.20 2003/10/20 04:29:17 rocky Exp $";
 
 #include <cdio/cdio.h>
 #include <cdio/sector.h>
@@ -56,33 +56,9 @@ static const char _rcsid[] = "$Id: _cdio_win32.c,v 1.17 2003/10/17 02:25:36 rock
 #include <sys/types.h>
 
 /* Win32 DeviceIoControl specifics */
-#ifndef MAXIMUM_NUMBER_TRACKS
-#    define MAXIMUM_NUMBER_TRACKS 100
-#endif
-typedef struct _TRACK_DATA {
-    UCHAR Format;
-    UCHAR Control : 4;
-    UCHAR Adr : 4;
-    UCHAR TrackNumber;
-    UCHAR Reserved1;
-    UCHAR Address[4];
-} TRACK_DATA, *PTRACK_DATA;
-typedef struct _CDROM_TOC {
-    UCHAR Length[2];
-    UCHAR FirstTrack;
-    UCHAR LastTrack;
-    TRACK_DATA TrackData[MAXIMUM_NUMBER_TRACKS];
-} CDROM_TOC, *PCDROM_TOC;
-typedef enum _TRACK_MODE_TYPE {
-    YellowMode2,
-    XAForm2,
-    CDDA
-} TRACK_MODE_TYPE, *PTRACK_MODE_TYPE;
-typedef struct __RAW_READ_INFO {
-    LARGE_INTEGER DiskOffset;
-    ULONG SectorCount;
-    TRACK_MODE_TYPE TrackMode;
-} RAW_READ_INFO, *PRAW_READ_INFO;
+/***** FIXME: #include ntddcdrm.h from Wine, but probably need to 
+   modify it a little.
+*/
 
 #ifndef IOCTL_CDROM_BASE
 #    define IOCTL_CDROM_BASE FILE_DEVICE_CD_ROM
@@ -96,6 +72,63 @@ typedef struct __RAW_READ_INFO {
                                       METHOD_OUT_DIRECT, FILE_READ_ACCESS)
 #endif
 
+#ifndef IOCTL_CDROM_READ_Q_CHANNEL
+#define IOCTL_CDROM_READ_Q_CHANNEL      CTL_CODE(IOCTL_CDROM_BASE, 0x000B, METHOD_BUFFERED, FILE_READ_ACCESS)
+#endif
+
+typedef struct _TRACK_DATA {
+    UCHAR Format;
+    UCHAR Control : 4;
+    UCHAR Adr : 4;
+    UCHAR TrackNumber;
+    UCHAR Reserved1;
+    UCHAR Address[4];
+} TRACK_DATA, *PTRACK_DATA;
+
+typedef struct _CDROM_TOC {
+    UCHAR Length[2];
+    UCHAR FirstTrack;
+    UCHAR LastTrack;
+    TRACK_DATA TrackData[CDIO_CD_MAX_TRACKS+1];
+} CDROM_TOC, *PCDROM_TOC;
+
+#define IOCTL_CDROM_SUB_Q_CHANNEL    0x00
+#define IOCTL_CDROM_CURRENT_POSITION 0x01
+#define IOCTL_CDROM_MEDIA_CATALOG    0x02
+#define IOCTL_CDROM_TRACK_ISRC       0x03
+
+typedef struct _CDROM_SUB_Q_DATA_FORMAT {
+    UCHAR               Format;
+    UCHAR               Track;
+} CDROM_SUB_Q_DATA_FORMAT, *PCDROM_SUB_Q_DATA_FORMAT;
+
+typedef struct _SUB_Q_HEADER {
+  UCHAR Reserved;
+  UCHAR AudioStatus;
+  UCHAR DataLength[2];
+} SUB_Q_HEADER, *PSUB_Q_HEADER;
+
+typedef struct _SUB_Q_MEDIA_CATALOG_NUMBER {
+  SUB_Q_HEADER Header;
+  UCHAR FormatCode;
+  UCHAR Reserved[3];
+  UCHAR Reserved1 : 7;
+  UCHAR Mcval :1;
+  UCHAR MediaCatalog[15];
+} SUB_Q_MEDIA_CATALOG_NUMBER, *PSUB_Q_MEDIA_CATALOG_NUMBER;
+
+typedef enum _TRACK_MODE_TYPE {
+    YellowMode2,
+    XAForm2,
+    CDDA
+} TRACK_MODE_TYPE, *PTRACK_MODE_TYPE;
+
+typedef struct __RAW_READ_INFO {
+    LARGE_INTEGER DiskOffset;
+    ULONG SectorCount;
+    TRACK_MODE_TYPE TrackMode;
+} RAW_READ_INFO, *PRAW_READ_INFO;
+
 /* Win32 aspi specific */
 #define WIN_NT               ( GetVersion() < 0x80000000 )
 #define ASPI_HAID           0
@@ -209,7 +242,7 @@ _cdio_mciSendCommand(int id, UINT msg, DWORD flags, void *arg)
     char error[256];
     
     mciGetErrorString(mci_error, error, 256);
-    cdio_error("mciSendCommand() error: %s", error);
+    cdio_warn("mciSendCommand() error: %s", error);
   }
   return(mci_error == 0);
 }
@@ -579,8 +612,17 @@ _cdio_mmc_read_sectors (void *user_data, void *data, lsn_t lsn,
 			 sizeof(RAW_READ_INFO), buf,
 			 sizeof(buf), &dwBytesReturned, NULL )
 	== 0 ) {
+      /* Retry in Yellowmode2 */
+      cdrom_raw.TrackMode = YellowMode2;
+      if( DeviceIoControl( _obj->h_device_handle,
+			 IOCTL_CDROM_RAW_READ, &cdrom_raw,
+			 sizeof(RAW_READ_INFO), buf,
+			 sizeof(buf), &dwBytesReturned, NULL )
+	  == 0 ) {
+	cdio_info("Error reading %lu (%ld)\n", lsn, GetLastError());
 	return 1;
       }
+    }
    }
 
   /* FIXME! remove the 8 (SUBHEADER size) below... */
@@ -644,9 +686,17 @@ _cdio_read_mode2_sector (void *user_data, void *data, lsn_t lsn,
 			 sizeof(RAW_READ_INFO), buf,
 			 sizeof(buf), &dwBytesReturned, NULL )
 	== 0 ) {
-      cdio_debug("Error reading %lu\n", lsn);
-      return 1;
+      /* Retry in Yellowmode2 */
+      cdrom_raw.TrackMode = YellowMode2;
+      if( DeviceIoControl( _obj->h_device_handle,
+			 IOCTL_CDROM_RAW_READ, &cdrom_raw,
+			 sizeof(RAW_READ_INFO), buf,
+			 sizeof(buf), &dwBytesReturned, NULL )
+	  == 0 ) {
+	cdio_info("Error reading %lu (%ld)\n", lsn, GetLastError());
+	return 1;
       }
+    }
     ret = 0;
   }
     
@@ -850,7 +900,7 @@ _cdio_read_toc (_img_private_t *_obj)
 			 IOCTL_CDROM_READ_TOC,
 			 NULL, 0, &cdrom_toc, sizeof(CDROM_TOC),
 			 &dwBytesReturned, NULL ) == 0 ) {
-      cdio_debug( "could not read TOCHDR" );
+      cdio_warn( "could not read TOCHDR" );
       return false;
     }
     
@@ -898,12 +948,12 @@ _cdio_eject_media (void *user_data) {
   i_flags = MCI_OPEN_TYPE | MCI_OPEN_TYPE_ID |
     MCI_OPEN_ELEMENT | MCI_OPEN_SHAREABLE;
   
-  if( !mciSendCommand( 0, MCI_OPEN, i_flags, (unsigned long)&op ) ) {
+  if( _cdio_mciSendCommand( 0, MCI_OPEN, i_flags, &op ) ) {
     st.dwItem = MCI_STATUS_READY;
     /* Eject disc */
-    ret = mciSendCommand( op.wDeviceID, MCI_SET, MCI_SET_DOOR_OPEN, 0 ) != 0;
+    ret = _cdio_mciSendCommand( op.wDeviceID, MCI_SET, MCI_SET_DOOR_OPEN, 0 ) != 0;
     /* Release access to the device */
-    mciSendCommand( op.wDeviceID, MCI_CLOSE, MCI_WAIT, 0 );
+    _cdio_mciSendCommand( op.wDeviceID, MCI_CLOSE, MCI_WAIT, 0 );
   } else 
     ret = 0;
   
@@ -945,6 +995,40 @@ _cdio_get_first_track_num(void *user_data)
   return _obj->first_track_num;
 }
 
+/*!
+  Return the media catalog number MCN.
+
+  Note: string is malloc'd so caller should free() then returned
+  string when done with it.
+
+ */
+static char *
+_cdio_get_mcn (void *env) {
+
+  _img_private_t *_obj = env;
+
+  if( ! _obj->hASPI ) {
+    DWORD dwBytesReturned;
+    SUB_Q_MEDIA_CATALOG_NUMBER mcn;
+    CDROM_SUB_Q_DATA_FORMAT q_data_format;
+    
+    memset( &mcn, 0, sizeof(mcn) );
+
+    q_data_format.Format = IOCTL_CDROM_MEDIA_CATALOG;
+    q_data_format.Track=1;
+    
+    if( DeviceIoControl( _obj->h_device_handle,
+			 IOCTL_CDROM_READ_Q_CHANNEL,
+			 &q_data_format, sizeof(q_data_format), 
+			 &mcn, sizeof(mcn),
+			 &dwBytesReturned, NULL ) == 0 ) {
+      cdio_warn( "could not read Q Channel at track 1");
+    } else if (mcn.Mcval)
+      return strdup(mcn.MediaCatalog);
+  }
+  return NULL;
+}
+
 /*!
   Return the number of tracks in the current medium.
   CDIO_INVALID_TRACK is returned on error.
@@ -986,15 +1070,14 @@ _cdio_get_track_format(void *env, track_t track_num)
     i_flags = MCI_OPEN_TYPE | MCI_OPEN_TYPE_ID |
       MCI_OPEN_ELEMENT | MCI_OPEN_SHAREABLE;
     
-    if( !_cdio_mciSendCommand( 0, MCI_OPEN, i_flags, &op ) ) {
+    if( _cdio_mciSendCommand( 0, MCI_OPEN, i_flags, &op ) ) {
       st.dwItem  = MCI_CDA_STATUS_TYPE_TRACK;
       st.dwTrack = track_num;
       i_flags = MCI_TRACK | MCI_STATUS_ITEM ;
-      ret = mciSendCommand( op.wDeviceID, MCI_STATUS, i_flags, 
-			    (unsigned long) &st );
+      ret = _cdio_mciSendCommand( op.wDeviceID, MCI_STATUS, i_flags, &st );
       
       /* Release access to the device */
-      mciSendCommand( op.wDeviceID, MCI_CLOSE, MCI_WAIT, 0 );
+      _cdio_mciSendCommand( op.wDeviceID, MCI_CLOSE, MCI_WAIT, 0 );
       
       switch(st.dwReturn) {
       case MCI_CDA_TRACK_AUDIO:
@@ -1033,9 +1116,9 @@ _cdio_get_track_format(void *env, track_t track_num)
   FIXME: there's gotta be a better design for this and get_track_format?
 */
 static bool
-_cdio_get_track_green(void *user_data, track_t track_num) 
+_cdio_get_track_green(void *env, track_t track_num) 
 {
-  _img_private_t *_obj = user_data;
+  _img_private_t *_obj = env;
   
   if (!_obj->toc_init) _cdio_read_toc (_obj) ;
 
@@ -1044,6 +1127,15 @@ _cdio_get_track_green(void *user_data, track_t track_num)
   if (track_num > _obj->total_tracks+1 || track_num == 0)
     return false;
 
+  switch (_cdio_get_track_format(env, track_num)) {
+  case TRACK_FORMAT_ERROR:
+  case TRACK_FORMAT_CDI:
+  case TRACK_FORMAT_AUDIO:
+    return false;
+  default:
+    break;
+  }
+
   /* FIXME: Dunno if this is the right way, but it's what 
      I was using in cd-info for a while.
    */
@@ -1058,9 +1150,9 @@ _cdio_get_track_green(void *user_data, track_t track_num)
   False is returned if there is no track entry.
 */
 static bool
-_cdio_get_track_msf(void *user_data, track_t track_num, msf_t *msf)
+_cdio_get_track_msf(void *env, track_t track_num, msf_t *msf)
 {
-  _img_private_t *_obj = user_data;
+  _img_private_t *_obj = env;
 
   if (NULL == msf) return false;
 
@@ -1189,7 +1281,7 @@ cdio_open_win32 (const char *source_name)
     .get_default_device = cdio_get_default_device_win32,
     .get_devices        = cdio_get_devices_win32,
     .get_first_track_num= _cdio_get_first_track_num,
-    .get_mcn            = NULL, 
+    .get_mcn            = _cdio_get_mcn, 
     .get_num_tracks     = _cdio_get_num_tracks,
     .get_track_format   = _cdio_get_track_format,
     .get_track_green    = _cdio_get_track_green,
