@@ -1,5 +1,5 @@
 /*
-    $Id: cd-info.c,v 1.43 2003/10/15 01:59:51 rocky Exp $
+    $Id: cd-info.c,v 1.47 2003/11/16 19:30:45 rocky Exp $
 
     Copyright (C) 2003 Rocky Bernstein <rocky@panix.com>
     Copyright (C) 1996,1997,1998  Gerd Knorr <kraxel@bytesex.org>
@@ -255,8 +255,10 @@ parse_options (int argc, const char *argv[])
 	exit (EXIT_FAILURE);
       }
       
-      if (opts.source_image == OP_SOURCE_DEVICE)
+      if (opts.source_image == IMAGE_DEVICE)
 	source_name = fillout_device_name(remaining_arg);
+      else 
+	source_name = strdup(remaining_arg);
       
       if ( (poptGetArgs(optCon)) != NULL) {
 	fprintf (stderr, 
@@ -491,9 +493,9 @@ print_iso9660_recurse (CdIo *cdio, const char pathname[], cdio_fs_anal_t fs,
   
   _CDIO_LIST_FOREACH (entnode, entlist)
     {
-      char *iso_name = _cdio_list_node_data (entnode);
+      iso9660_stat_t *statbuf = _cdio_list_node_data (entnode);
+      char *iso_name = statbuf->filename;
       char _fullname[4096] = { 0, };
-      iso9660_stat_t statbuf;
       char translated_name[MAX_ISONAME+1];
 #define DATESTR_SIZE 30
       char date_str[DATESTR_SIZE];
@@ -503,37 +505,31 @@ print_iso9660_recurse (CdIo *cdio, const char pathname[], cdio_fs_anal_t fs,
       snprintf (_fullname, sizeof (_fullname), "%s%s", pathname, 
 		iso_name);
   
-      if (iso9660_fs_stat (cdio, _fullname, &statbuf, is_mode2)) {
-        fprintf(stderr, "Error getting file info for directory %s/\n", 
-		_fullname);
-	continue;
-      }
-
       strncat (_fullname, "/", sizeof (_fullname));
 
-      if (statbuf.type == _STAT_DIR
+      if (statbuf->type == _STAT_DIR
           && strcmp (iso_name, ".") 
           && strcmp (iso_name, ".."))
         _cdio_list_append (dirlist, strdup (_fullname));
 
       if (fs & CDIO_FS_ANAL_XA) {
 	printf ( "  %c %s %d %d [fn %.2d] [LSN %6lu] ",
-		 (statbuf.type == _STAT_DIR) ? 'd' : '-',
-		 iso9660_get_xa_attr_str (statbuf.xa.attributes),
-		 uint16_from_be (statbuf.xa.user_id),
-		 uint16_from_be (statbuf.xa.group_id),
-		 statbuf.xa.filenum,
-		 (long unsigned int) statbuf.lsn);
+		 (statbuf->type == _STAT_DIR) ? 'd' : '-',
+		 iso9660_get_xa_attr_str (statbuf->xa.attributes),
+		 uint16_from_be (statbuf->xa.user_id),
+		 uint16_from_be (statbuf->xa.group_id),
+		 statbuf->xa.filenum,
+		 (long unsigned int) statbuf->lsn);
 	
-	if (uint16_from_be(statbuf.xa.attributes) & XA_ATTR_MODE2FORM2) {
+	if (uint16_from_be(statbuf->xa.attributes) & XA_ATTR_MODE2FORM2) {
 	  printf ("%9u (%9u)",
-		  (unsigned int) statbuf.secsize * M2F2_SECTOR_SIZE,
-		  (unsigned int) statbuf.size);
+		  (unsigned int) statbuf->secsize * M2F2_SECTOR_SIZE,
+		  (unsigned int) statbuf->size);
 	} else {
-	  printf ("%9u", (unsigned int) statbuf.size);
+	  printf ("%9u", (unsigned int) statbuf->size);
 	}
       }
-      strftime(date_str, DATESTR_SIZE, "%b %e %Y %H:%M ", &statbuf.tm);
+      strftime(date_str, DATESTR_SIZE, "%b %e %Y %H:%M ", &statbuf->tm);
       printf (" %s %s\n", date_str, translated_name);
     }
 
@@ -591,7 +587,7 @@ print_iso9660_fs (CdIo *cdio, cdio_fs_anal_t fs, track_format_t track_format)
 }
 
 static void
-print_analysis(int ms_offset, cdio_analysis_t cdio_analysis, 
+print_analysis(int ms_offset, cdio_iso_analysis_t cdio_iso_analysis, 
 	       cdio_fs_anal_t fs, int first_data, unsigned int num_audio, 
 	       track_t num_tracks, track_t first_track_num, 
 	       track_format_t track_format, CdIo *cdio)
@@ -611,7 +607,7 @@ print_analysis(int ms_offset, cdio_analysis_t cdio_analysis,
   case CDIO_FS_ISO_9660:
     printf("CD-ROM with ISO 9660 filesystem");
     if (fs & CDIO_FS_ANAL_JOLIET) {
-      printf(" and joliet extension level %d", cdio_analysis.joliet_level);
+      printf(" and joliet extension level %d", cdio_iso_analysis.joliet_level);
     }
     if (fs & CDIO_FS_ANAL_ROCKRIDGE)
       printf(" and rockridge extensions");
@@ -650,7 +646,7 @@ print_analysis(int ms_offset, cdio_analysis_t cdio_analysis,
   case CDIO_FS_ISO_9660_INTERACTIVE:
   case CDIO_FS_ISO_HFS:
     printf("ISO 9660: %i blocks, label `%.32s'\n",
-	   cdio_analysis.isofs_size, cdio_analysis.iso_label);
+	   cdio_iso_analysis.isofs_size, cdio_iso_analysis.iso_label);
     if (opts.print_iso9660) 
       print_iso9660_fs(cdio, fs, track_format);
     break;
@@ -731,22 +727,22 @@ int
 main(int argc, const char *argv[])
 {
 
-  CdIo          *cdio=NULL;
-  cdio_fs_anal_t fs=CDIO_FS_AUDIO;
+  CdIo               *cdio=NULL;
+  cdio_fs_anal_t      fs=CDIO_FS_AUDIO;
   int i;
-  lsn_t          start_track_lsn;      /* lsn of first track */
-  lsn_t          data_start =0;        /* start of data area */
-  int            ms_offset = 0;
-  track_t        num_tracks=0;
-  track_t        first_track_num  =  0;
-  unsigned int   num_audio   =  0;  /* # of audio tracks */
-  unsigned int   num_data    =  0;  /* # of data tracks */
-  int            first_data  = -1;  /* # of first data track */
-  int            first_audio = -1;  /* # of first audio track */
-  cdio_analysis_t cdio_analysis; 
-  char           *media_catalog_number;  
+  lsn_t               start_track_lsn;      /* lsn of first track */
+  lsn_t               data_start =0;        /* start of data area */
+  int                 ms_offset = 0;
+  track_t             num_tracks=0;
+  track_t             first_track_num  =  0;
+  unsigned int        num_audio   =  0;  /* # of audio tracks */
+  unsigned int        num_data    =  0;  /* # of data tracks */
+  int                 first_data  = -1;  /* # of first data track */
+  int                 first_audio = -1;  /* # of first audio track */
+  cdio_iso_analysis_t cdio_iso_analysis; 
+  char                *media_catalog_number;  
       
-  memset(&cdio_analysis, 0, sizeof(cdio_analysis));
+  memset(&cdio_iso_analysis, 0, sizeof(cdio_iso_analysis));
   init();
 
   /* Parse our arguments; every option seen by `parse_opt' will
@@ -755,6 +751,18 @@ main(int argc, const char *argv[])
      
   print_version(program_name, VERSION, opts.no_header, opts.version_only);
 
+  if (opts.debug_level == 3) {
+    cdio_loglevel_default = CDIO_LOG_INFO;
+#ifdef HAVE_VCDINFO
+    vcd_loglevel_default = VCD_LOG_INFO;
+#endif
+  } else if (opts.debug_level >= 4) {
+    cdio_loglevel_default = CDIO_LOG_DEBUG;
+#ifdef HAVE_VCDINFO
+    vcd_loglevel_default = VCD_LOG_DEBUG;
+#endif
+  }
+
   switch (opts.source_image) {
   case IMAGE_UNKNOWN:
   case IMAGE_AUTO:
@@ -942,7 +950,7 @@ main(int argc, const char *argv[])
       
       /* CD-I/Ready says start_track_lsn <= 30*75 then CDDA */
       if (start_track_lsn > 100 /* 100 is just a guess */) {
-	fs = cdio_guess_cd_type(cdio, 0, 1, &cdio_analysis);
+	fs = cdio_guess_cd_type(cdio, 0, 1, &cdio_iso_analysis);
 	if ((CDIO_FSTYPE(fs)) != CDIO_FS_UNKNOWN)
 	  fs |= CDIO_FS_ANAL_HIDDEN_TRACK;
 	else {
@@ -952,7 +960,7 @@ main(int argc, const char *argv[])
 		 (long unsigned int) start_track_lsn);
 	}
       }
-      print_analysis(ms_offset, cdio_analysis, fs, first_data, num_audio,
+      print_analysis(ms_offset, cdio_iso_analysis, fs, first_data, num_audio,
 		     num_tracks, first_track_num, 
 		     cdio_get_track_format(cdio, 1), cdio);
     } else {
@@ -983,10 +991,10 @@ main(int argc, const char *argv[])
 	  data_start = start_track_lsn;
 	
 	/* skip tracks which belong to the current walked session */
-	if (start_track_lsn < data_start + cdio_analysis.isofs_size)
+	if (start_track_lsn < data_start + cdio_iso_analysis.isofs_size)
 	  continue;
 	
-	fs = cdio_guess_cd_type(cdio, start_track_lsn, i, &cdio_analysis);
+	fs = cdio_guess_cd_type(cdio, start_track_lsn, i, &cdio_iso_analysis);
 
 	if (i > 1) {
 	  /* track is beyond last session -> new session found */
@@ -994,13 +1002,14 @@ main(int argc, const char *argv[])
 	  printf("session #%d starts at track %2i, LSN: %lu,"
 		 " ISO 9660 blocks: %6i\n",
 		 j++, i, (unsigned long int) start_track_lsn, 
-		 cdio_analysis.isofs_size);
+		 cdio_iso_analysis.isofs_size);
 	  printf("ISO 9660: %i blocks, label `%.32s'\n",
-		 cdio_analysis.isofs_size, cdio_analysis.iso_label);
+		 cdio_iso_analysis.isofs_size, cdio_iso_analysis.iso_label);
 	  fs |= CDIO_FS_ANAL_MULTISESSION;
 	} else {
-	  print_analysis(ms_offset, cdio_analysis, fs, first_data, num_audio,
-			 num_tracks, first_track_num, track_format, cdio);
+	  print_analysis(ms_offset, cdio_iso_analysis, fs, first_data, 
+			 num_audio, num_tracks, first_track_num, 
+			 track_format, cdio);
 	}
 
 	if ( !(CDIO_FSTYPE(fs) == CDIO_FS_ISO_9660 ||
